@@ -1,8 +1,8 @@
 """
-UBX Protocol Input payload definitions
+UBX Protocol SET payload definitions.
 
 THESE ARE THE PAYLOAD DEFINITIONS FOR _SET_ MESSAGES _TO_ THE RECEIVER
-(e.g. configuration and calibration commands; AssistNow payloads)
+(e.g. configuration and calibration commands; AssistNow payloads).
 
 Created on 27 Sep 2020
 
@@ -10,6 +10,7 @@ Information sourced from public domain u-blox Interface Specifications © 2013-2
 
 :author: semuadmin
 """
+
 # pylint: disable=too-many-lines, line-too-long
 
 from pyubx2.ubxtypes_core import (
@@ -52,7 +53,7 @@ UBX_PAYLOADS_SET = {
             },
         ),
     },
-    "ALP-ALPSRV": {
+    "AID-ALPSRV": {  # ALP server sends AlmanacPlus data to client
         "idSize": U1,
         "type": U1,
         "ofs": U2,
@@ -61,7 +62,7 @@ UBX_PAYLOADS_SET = {
         "dataSize": U2,
         "id1": U1,
         "id2": U1,
-        "id3": U1,
+        "id3": U4,
         "group": (
             "dataSize",
             {
@@ -231,6 +232,15 @@ UBX_PAYLOADS_SET = {
         ),
     },
     "CFG-ODO": UBX_GET["CFG-ODO"],
+    "CFG-OTP": {
+        "cfgHeader": U12,
+        "group": (
+            "None",
+            {
+                "cfgData": U1,
+            },
+        ),
+    },
     "CFG-PM2": UBX_GET["CFG-PM2"],
     "CFG-PMS": UBX_GET["CFG-PMS"],
     "CFG-PRT": UBX_GET["CFG-PRT"],
@@ -679,7 +689,7 @@ UBX_PAYLOADS_SET = {
         "omega": [I4, 2**-31],
         "omegaDot": [I4, 2**-43],
         "idot": [I2, 2**-43],
-        "reserved2": U4,
+        "reserved2": U2,
     },
     "MGA-GPS-HEALTH": {
         "type": U1,
