@@ -7,34 +7,42 @@ Created on 3 Oct 2020
 
 @author: semuadmin
 """
+
 # pylint: disable=line-too-long, invalid-name, missing-docstring, no-member
 
 import os
 import unittest
-import unittest
+from datetime import datetime
 
-from pyubx2 import UBXMessage, UBXReader, UBX_CLASSES, POLL
 import pyubx2.ubxtypes_core as ubt
+from pyubx2 import POLL, SET, UBX_CLASSES, UBXMessage, UBXReader
 from pyubx2.ubxhelpers import (
-    calc_checksum,
-    isvalid_checksum,
-    key_from_val,
-    get_bits,
-    itow2utc,
-    gnss2str,
-    dop2str,
-    gpsfix2str,
-    msgstr2bytes,
-    val2bytes,
-    bytes2val,
-    cfgkey2name,
-    cfgname2key,
-    protocol,
-    hextable,
+    attsiz,
     att2idx,
     att2name,
+    bytes2val,
+    calc_checksum,
     cel2cart,
+    cfgkey2name,
+    cfgname2key,
+    dop2str,
     escapeall,
+    get_bits,
+    getinputmode,
+    gnss2str,
+    gpsfix2str,
+    hextable,
+    isvalid_checksum,
+    itow2utc,
+    key_from_val,
+    msgstr2bytes,
+    process_monver,
+    protocol,
+    utc2itow,
+    val2bytes,
+    val2sphp,
+    val2twoscomp,
+    val2signmag,
 )
 
 
@@ -144,7 +152,7 @@ class StaticTest(unittest.TestCase):
 
     def testKeyfromVal(self):
         res = key_from_val(UBX_CLASSES, "MON")
-        self.assertEqual(res, (b"\x0A"))
+        self.assertEqual(res, (b"\x0a"))
 
     def testCalcChecksum(self):
         res = calc_checksum(b"\x06\x01\x02\x00\xf0\x05")
@@ -162,6 +170,11 @@ class StaticTest(unittest.TestCase):
         res = str(itow2utc(387092000))
         self.assertEqual(res, "11:31:14")
 
+    def testutc2itow(self):
+        dt = datetime(2024, 2, 8, 11, 31, 14)
+        res = utc2itow(dt)
+        self.assertEqual(res, (2300, 387092000))
+
     def testgnss2str(self):
         GNSS = {
             0: "GPS",
@@ -179,15 +192,25 @@ class StaticTest(unittest.TestCase):
             self.assertEqual(res, GNSS[i])
 
     def testgps2str(self):
-        fixs = ["NO FIX", "DR", "2D", "3D", "GPS + DR", "TIME ONLY"]
-        for i, fix in enumerate(range(0, 6)):
+        fixs = ["NO FIX", "DR", "2D", "3D", "GPS + DR", "TIME ONLY", "6"]
+        for i, fix in enumerate(range(0, 7)):
             res = gpsfix2str(fix)
             self.assertEqual(res, fixs[i])
 
     def testdop2str(self):
-        dops = ["Ideal", "Excellent", "Good", "Moderate", "Fair", "Poor"]
+        dops = [
+            "N/A",
+            "Ideal",
+            "Ideal",
+            "Excellent",
+            "Excellent",
+            "Good",
+            "Moderate",
+            "Fair",
+            "Poor",
+        ]
         i = 0
-        for dop in (1, 2, 5, 10, 20, 30):
+        for dop in (0, 0.9, 1, 1.4, 2, 5, 10, 20, 30):
             res = dop2str(dop)
             self.assertEqual(res, dops[i])
             i += 1
@@ -246,9 +269,13 @@ class StaticTest(unittest.TestCase):
         res = hextable(b"$GNGLL,5327.04319,S,00214.41396,E,223232.00,A,A*68\r\n", 8)
         self.assertEqual(res, EXPECTED_RESULT)
 
+    def testattsiz(self):  # test attsiz
+        self.assertEqual(attsiz("CH"), -1)
+        self.assertEqual(attsiz("C032"), 32)
+
     def testatt2idx(self):  # test att2idx
-        EXPECTED_RESULT = [4, 16, 101, 0]
-        atts = ["svid_04", "gnssId_16", "cno_101", "gmsLon"]
+        EXPECTED_RESULT = [4, 16, 101, 0, (3, 6), 0]
+        atts = ["svid_04", "gnssId_16", "cno_101", "gmsLon", "gnod_03_06", "dodgy_xx"]
         for i, att in enumerate(atts):
             res = att2idx(att)
             # print(res)
@@ -276,6 +303,65 @@ class StaticTest(unittest.TestCase):
         print(res)
         self.assertEqual(res, EXPECTED_RESULT)
 
+    def testval2sphp(self):
+        res = val2sphp(100.123456789)
+        self.assertEqual(res, (1001234567, 89))
+        res = val2sphp(-13.987654321)
+        self.assertEqual(res, (-139876543, -21))
+        res = val2sphp(5.9876543)
+        self.assertEqual(res, (59876543, 0))
+
+    def testgetinputmode(self):
+        res = getinputmode(UBXMessage("CFG", "CFG-ODO", POLL).serialize())
+        self.assertEqual(res, POLL)
+        res = getinputmode(
+            UBXMessage.config_poll(0, 0, ["CFG_UART1_BAUDRATE", 0x40530001]).serialize()
+        )
+        self.assertEqual(res, POLL)
+        res = getinputmode(
+            UBXMessage.config_set(
+                0, 0, [("CFG_UART1_BAUDRATE", 9600), (0x40530001, 115200)]
+            ).serialize()
+        )
+        self.assertEqual(res, SET)
+        res = getinputmode(
+            UBXMessage.config_del(0, 0, ["CFG_UART1_BAUDRATE", 0x40530001]).serialize()
+        )
+        self.assertEqual(res, SET)
+        res = getinputmode(UBXMessage("CFG", "CFG-INF", POLL, protocolID=1).serialize())
+        self.assertEqual(res, POLL)
+        res = getinputmode(
+            UBXMessage(
+                "CFG", "CFG-INF", SET, protocolID=1, infMsgMask_01=1, infMsgMask_02=1
+            ).serialize()
+        )
+        self.assertEqual(res, SET)
+
+    def testprocess_monver(self):
+        MONVER = b"\xb5\x62\x0a\x04\xdc\x00\x45\x58\x54\x20\x43\x4f\x52\x45\x20\x31\x2e\x30\x30\x20\x28\x66\x31\x37\x30\x36\x37\x29\x00\x00\x00\x00\x00\x00\x00\x00\x30\x30\x31\x39\x30\x30\x30\x30\x00\x00\x52\x4f\x4d\x20\x42\x41\x53\x45\x20\x30\x78\x31\x31\x38\x42\x32\x30\x36\x30\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x46\x57\x56\x45\x52\x3d\x48\x50\x47\x20\x31\x2e\x35\x30\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x50\x52\x4f\x54\x56\x45\x52\x3d\x32\x37\x2e\x35\x30\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x4d\x4f\x44\x3d\x5a\x45\x44\x2d\x46\x39\x50\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x47\x50\x53\x3b\x47\x4c\x4f\x3b\x47\x41\x4c\x3b\x42\x44\x53\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x53\x42\x41\x53\x3b\x51\x5a\x53\x53\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xce\x8b"
+        msg = UBXReader.parse(MONVER)
+        EXPECTED_RESULT = {
+            "swversion": "Flash 1.00 (f17067)",
+            "hwversion": "ZED-F9P 00190000",
+            "fwversion": "HPG 1.50",
+            "romversion": "27.50",
+            "gnss": "GPS GLO GAL BDS SBAS QZSS ",
+        }
+        res = process_monver(msg)
+        self.assertEqual(res, EXPECTED_RESULT)
+
+    def testval2twoscomp(self):
+        res = val2twoscomp(10, "U24")
+        self.assertEqual(res, 0b0000000000000000000001010)
+        res = val2twoscomp(-10, "U24")
+        self.assertEqual(res, 0b111111111111111111110110)
+
+    def testval2signmag(self):
+        res = val2signmag(10, "U24")
+        self.assertEqual(res, 0b0000000000000000000001010)
+        res = val2signmag(-10, "U24")
+        self.assertEqual(res, 0b1000000000000000000001010)
+
 
 if __name__ == "__main__":
     # import sys;sys.argv = ['', 'Test.testName']
